# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymaid/rpc/pb/pymaid.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name='pymaid/rpc/pb/pymaid.proto',
    package='pymaid.rpc.pb',
    syntax='proto3',
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1apymaid/rpc/pb/pymaid.proto\x12\rpymaid.rpc.pb\"\xb1\x03\n\x07\x43ontext\x12\x17\n\x0ftransmission_id\x18\x01 \x01(\r\x12\x36\n\x0bpacket_type\x18\x02 \x01(\x0e\x32!.pymaid.rpc.pb.Context.PacketType\x12\x37\n\x0cpacket_flags\x18\x03 \x01(\x0e\x32!.pymaid.rpc.pb.Context.PacketFlag\x12\x31\n\x08priority\x18\x04 \x01(\x0e\x32\x1f.pymaid.rpc.pb.Context.Priority\x12\x16\n\x0eservice_method\x18\x05 \x01(\t\x12\x14\n\x0cis_cancelled\x18\x06 \x01(\x08\x12\x11\n\tis_failed\x18\x07 \x01(\x08\x12\x14\n\x0cpayload_size\x18\x08 \x01(\r\"4\n\nPacketType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07REQUEST\x10\x01\x12\x0c\n\x08RESPONSE\x10\x02\"4\n\nPacketFlag\x12\x08\n\x04NULL\x10\x00\x12\x07\n\x03NEW\x10\x01\x12\n\n\x06\x43\x41NCEL\x10\x02\x12\x07\n\x03\x45ND\x10\x04\"&\n\x08Priority\x12\x07\n\x03LOW\x10\x00\x12\x07\n\x03MID\x10\x01\x12\x08\n\x04HIGH\x10\x02\"\x08\n\x06RpcAck\"\x06\n\x04Void\";\n\x0c\x45rrorMessage\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\tb\x06proto3'
)


_CONTEXT_PACKETTYPE = _descriptor.EnumDescriptor(
    name='PacketType',
    full_name='pymaid.rpc.pb.Context.PacketType',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN', index=0, number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='REQUEST', index=1, number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='RESPONSE', index=2, number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=333,
    serialized_end=385,
)
_sym_db.RegisterEnumDescriptor(_CONTEXT_PACKETTYPE)

_CONTEXT_PACKETFLAG = _descriptor.EnumDescriptor(
    name='PacketFlag',
    full_name='pymaid.rpc.pb.Context.PacketFlag',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='NULL', index=0, number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='NEW', index=1, number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CANCEL', index=2, number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='END', index=3, number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=387,
    serialized_end=439,
)
_sym_db.RegisterEnumDescriptor(_CONTEXT_PACKETFLAG)

_CONTEXT_PRIORITY = _descriptor.EnumDescriptor(
    name='Priority',
    full_name='pymaid.rpc.pb.Context.Priority',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='LOW', index=0, number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='MID', index=1, number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='HIGH', index=2, number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=441,
    serialized_end=479,
)
_sym_db.RegisterEnumDescriptor(_CONTEXT_PRIORITY)


_CONTEXT = _descriptor.Descriptor(
    name='Context',
    full_name='pymaid.rpc.pb.Context',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='transmission_id', full_name='pymaid.rpc.pb.Context.transmission_id', index=0,
            number=1, type=13, cpp_type=3, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='packet_type', full_name='pymaid.rpc.pb.Context.packet_type', index=1,
            number=2, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='packet_flags', full_name='pymaid.rpc.pb.Context.packet_flags', index=2,
            number=3, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='priority', full_name='pymaid.rpc.pb.Context.priority', index=3,
            number=4, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='service_method', full_name='pymaid.rpc.pb.Context.service_method', index=4,
            number=5, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_cancelled', full_name='pymaid.rpc.pb.Context.is_cancelled', index=5,
            number=6, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_failed', full_name='pymaid.rpc.pb.Context.is_failed', index=6,
            number=7, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='payload_size', full_name='pymaid.rpc.pb.Context.payload_size', index=7,
            number=8, type=13, cpp_type=3, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
        _CONTEXT_PACKETTYPE,
        _CONTEXT_PACKETFLAG,
        _CONTEXT_PRIORITY,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=46,
    serialized_end=479,
)


_RPCACK = _descriptor.Descriptor(
    name='RpcAck',
    full_name='pymaid.rpc.pb.RpcAck',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=481,
    serialized_end=489,
)


_VOID = _descriptor.Descriptor(
    name='Void',
    full_name='pymaid.rpc.pb.Void',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=491,
    serialized_end=497,
)


_ERRORMESSAGE = _descriptor.Descriptor(
    name='ErrorMessage',
    full_name='pymaid.rpc.pb.ErrorMessage',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='code', full_name='pymaid.rpc.pb.ErrorMessage.code', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='message', full_name='pymaid.rpc.pb.ErrorMessage.message', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='data', full_name='pymaid.rpc.pb.ErrorMessage.data', index=2,
            number=3, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=499,
    serialized_end=558,
)

_CONTEXT.fields_by_name['packet_type'].enum_type = _CONTEXT_PACKETTYPE
_CONTEXT.fields_by_name['packet_flags'].enum_type = _CONTEXT_PACKETFLAG
_CONTEXT.fields_by_name['priority'].enum_type = _CONTEXT_PRIORITY
_CONTEXT_PACKETTYPE.containing_type = _CONTEXT
_CONTEXT_PACKETFLAG.containing_type = _CONTEXT
_CONTEXT_PRIORITY.containing_type = _CONTEXT
DESCRIPTOR.message_types_by_name['Context'] = _CONTEXT
DESCRIPTOR.message_types_by_name['RpcAck'] = _RPCACK
DESCRIPTOR.message_types_by_name['Void'] = _VOID
DESCRIPTOR.message_types_by_name['ErrorMessage'] = _ERRORMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Context = _reflection.GeneratedProtocolMessageType('Context', (_message.Message,), {
    'DESCRIPTOR': _CONTEXT,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.Context)
})
_sym_db.RegisterMessage(Context)

RpcAck = _reflection.GeneratedProtocolMessageType('RpcAck', (_message.Message,), {
    'DESCRIPTOR': _RPCACK,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.RpcAck)
})
_sym_db.RegisterMessage(RpcAck)

Void = _reflection.GeneratedProtocolMessageType('Void', (_message.Message,), {
    'DESCRIPTOR': _VOID,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.Void)
})
_sym_db.RegisterMessage(Void)

ErrorMessage = _reflection.GeneratedProtocolMessageType('ErrorMessage', (_message.Message,), {
    'DESCRIPTOR': _ERRORMESSAGE,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.ErrorMessage)
})
_sym_db.RegisterMessage(ErrorMessage)


# @@protoc_insertion_point(module_scope)
